#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

/**************************************************************************
~~~~~~~~~~~~~~~~~~~~~~~~ Disclaimer ~~~~~~~~~~~~~~~~~~~~~~~~~~~
*************************************************************************
* THIS SOFTWARE IS PROVIDED BY THE THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*************************************************************************/

namespace NinjaTrader.Indicator{

    [Description("SuperTrend")]
    public class SuperTrend : Indicator
	{
            private int lenght=5;
            private double multiplier=3;
			private BoolSeries Trend;

        protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.White, 2), PlotStyle.Line, "superTrend"));
			CalculateOnBarClose = false;
            Overlay				= true;
           	Trend=new BoolSeries(this);
        }

		public override string ToString(){return "SuperTrend";}
		
        protected override void OnBarUpdate()
		{
			if(CurrentBar<lenght)
			{ 
				Trend.Set(true);
				return; 
			}
			if(Input[0]>superTrend[1]) Trend.Set(true);
			else if(Input[0]<superTrend[1]) Trend.Set(false);
			else Trend.Set(Trend[1]);
			
			if(Trend[0] && !Trend[1]) superTrend.Set(Close[0]-ATR(Lenght)[0]*Multiplier);
			else if(!Trend[0] && Trend[1]) superTrend.Set(Close[0]+ATR(Lenght)[0]*Multiplier);
			else superTrend.Set(Trend[0] && (Close[0]-ATR(Lenght)[0]*Multiplier)>superTrend[1]?(Close[0]-ATR(Lenght)[0]*Multiplier):!Trend[0] && (Close[0]+ATR(Lenght)[0]*Multiplier)<superTrend[1]?(Close[0]+ATR(Lenght)[0]*Multiplier):superTrend[1]);
		}
		
        #region Properties
        [Browsable(false)]
        [XmlIgnore()]
        public DataSeries superTrend{get{return Values[0];}}
		
        [Category("Parameters")]
		[NinjaTrader.Gui.Design.DisplayName("ATR period")]
        public int Lenght{
            get { return lenght; }
            set { lenght = Math.Max(1, value); }
        }
        [Category("Parameters")]
        [NinjaTrader.Gui.Design.DisplayName("Multiplier")]
		public double Multiplier{
            get { return multiplier; }
            set { multiplier = Math.Max(0.0001, value); }
        }
        #endregion
    }
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SuperTrend[] cacheSuperTrend = null;

        private static SuperTrend checkSuperTrend = new SuperTrend();

        /// <summary>
        /// SuperTrend
        /// </summary>
        /// <returns></returns>
        public SuperTrend SuperTrend(int lenght, double multiplier)
        {
            return SuperTrend(Input, lenght, multiplier);
        }

        /// <summary>
        /// SuperTrend
        /// </summary>
        /// <returns></returns>
        public SuperTrend SuperTrend(Data.IDataSeries input, int lenght, double multiplier)
        {
            if (cacheSuperTrend != null)
                for (int idx = 0; idx < cacheSuperTrend.Length; idx++)
                    if (cacheSuperTrend[idx].Lenght == lenght && Math.Abs(cacheSuperTrend[idx].Multiplier - multiplier) <= double.Epsilon && cacheSuperTrend[idx].EqualsInput(input))
                        return cacheSuperTrend[idx];

            lock (checkSuperTrend)
            {
                checkSuperTrend.Lenght = lenght;
                lenght = checkSuperTrend.Lenght;
                checkSuperTrend.Multiplier = multiplier;
                multiplier = checkSuperTrend.Multiplier;

                if (cacheSuperTrend != null)
                    for (int idx = 0; idx < cacheSuperTrend.Length; idx++)
                        if (cacheSuperTrend[idx].Lenght == lenght && Math.Abs(cacheSuperTrend[idx].Multiplier - multiplier) <= double.Epsilon && cacheSuperTrend[idx].EqualsInput(input))
                            return cacheSuperTrend[idx];

                SuperTrend indicator = new SuperTrend();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Lenght = lenght;
                indicator.Multiplier = multiplier;
                Indicators.Add(indicator);
                indicator.SetUp();

                SuperTrend[] tmp = new SuperTrend[cacheSuperTrend == null ? 1 : cacheSuperTrend.Length + 1];
                if (cacheSuperTrend != null)
                    cacheSuperTrend.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSuperTrend = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// SuperTrend
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SuperTrend SuperTrend(int lenght, double multiplier)
        {
            return _indicator.SuperTrend(Input, lenght, multiplier);
        }

        /// <summary>
        /// SuperTrend
        /// </summary>
        /// <returns></returns>
        public Indicator.SuperTrend SuperTrend(Data.IDataSeries input, int lenght, double multiplier)
        {
            return _indicator.SuperTrend(input, lenght, multiplier);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// SuperTrend
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SuperTrend SuperTrend(int lenght, double multiplier)
        {
            return _indicator.SuperTrend(Input, lenght, multiplier);
        }

        /// <summary>
        /// SuperTrend
        /// </summary>
        /// <returns></returns>
        public Indicator.SuperTrend SuperTrend(Data.IDataSeries input, int lenght, double multiplier)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SuperTrend(input, lenght, multiplier);
        }
    }
}
#endregion
